package com.zhentao.user.controller;

import com.zhentao.config.NullLogin;
import com.zhentao.user.service.UserLoginService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpSession;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Base64;
import java.util.Date;

@Controller
public class VerificationController {

    @Autowired
    private UserLoginService userService;
    @Value("${verification.secretKey}")
    private String secretKey;

    @Value("${verification.expiryMinutes}")
    private long expiryMinutes;

    @GetMapping("/verify-email")
    @NullLogin
    public String verifyEmail(@RequestParam("email") String email,
                              @RequestParam("timestamp") long timestamp,
                              @RequestParam("signature") String signature,
                              HttpSession session,
                              Model model, RedirectAttributes redirectAttributes) {
        // 验证链接是否过期
        long currentTime = System.currentTimeMillis();
        long linkAgeMinutes = (currentTime - timestamp) / (1000 * 60);

        if (!isValidVerificationLink(email, timestamp, signature)) {
            model.addAttribute("errorMessage", "无效或过期的验证链接");
            return "verification-error"; // 对应验证错误页面的视图名称
        }
        session.setAttribute("verifiedEmail", email);
        session.setAttribute("verificationTime", new Date());
        // 签名验证通过，标记邮箱为已验证
        model.addAttribute("email", email);
        return "registration-form";
    }
    private boolean isValidVerificationLink(String email, long timestamp, String signature) {
        // 验证时间戳
        long currentTime = System.currentTimeMillis();
        if ((currentTime - timestamp) / (1000 * 60) > expiryMinutes) {
            return false;
        }

        // 验证签名
        String expectedSignature = generateSignature(email, timestamp);
        return expectedSignature.equals(signature);
    }
    private String generateSignature(String email, long timestamp) {
        try {
            MessageDigest digest = MessageDigest.getInstance("SHA-256");
            String content = email + timestamp + secretKey;
            byte[] hash = digest.digest(content.getBytes(StandardCharsets.UTF_8));
            return Base64.getUrlEncoder().withoutPadding().encodeToString(hash);
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException("生成签名失败", e);
        }
    }
}