package com.zhentao.user.service.impl;

import com.zhentao.user.service.EmailService;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Service;

import javax.mail.internet.MimeMessage;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Base64;

@Service
public class EmailServiceImpl implements EmailService {
    @Autowired
    private JavaMailSender mailSender;
    @Value("${verification.secretKey}")
    private String secretKey;

    @SneakyThrows
    @Override
    public void sendVerificationEmail(String to) {
        MimeMessage message = mailSender.createMimeMessage();
        MimeMessageHelper helper = new MimeMessageHelper(message, true, "UTF-8");
        // 生成带签名和时间戳的验证URL
        long timestamp = System.currentTimeMillis();
        String signature = generateSignature(to, timestamp);
        String verifyUrl = "http://47.111.130.63:8081/verify-email" +
                "?email=" + to +
                "&timestamp=" + timestamp +
                "&signature=" + signature;
        helper.setFrom("vx3181048507@126.com");
        helper.setTo(to);
        helper.setSubject("邮箱验证");
        String htmlContent = "<html><body>" +
                "<h3>请点击以下链接验证您的邮箱：</h3>" +
                "<a href=\""+verifyUrl + "\">" +"点击即可注册"+ "</a>" +
                "<p>链接有效期为30分钟，请尽快完成验证。</p>" +
                "</body></html>";
        helper.setText(htmlContent,true);
        mailSender.send(message);
    }
    private String generateSignature(String email, long timestamp) {
        try {
            MessageDigest digest = MessageDigest.getInstance("SHA-256");
            String content = email + timestamp + secretKey;
            byte[] hash = digest.digest(content.getBytes(StandardCharsets.UTF_8));
            return Base64.getUrlEncoder().withoutPadding().encodeToString(hash);
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException("生成签名失败", e);
        }
    }
}
